# coding: utf-8
import argparse
import logging
#logging.basicConfig(level=logging.DEBUG)
from os import system, environ
from pathlib import Path

from transpiler import transpile
from transpiler.format import format_code

# load .env file
from dotenv import load_dotenv

load_dotenv()

# todo: promise https://lab.nexedi.com/nexedi/slapos.toolbox/blob/master/slapos/promise/plugin/check_socket_listening.py
# todo: scan fs https://lab.nexedi.com/xavier_thompson/scan-filesystem/blob/master/rust/src/main.rs
# todo: refs https://lab.nexedi.com/xavier_thompson/typon-snippets/tree/master/references

parser = argparse.ArgumentParser()
parser.add_argument("-c", "--compile", action="store_true")
parser.add_argument("-g", "--generate", action="store_true")
parser.add_argument("-o", "--only", nargs="+")
args = parser.parse_args()


def run_tests():
    for path in Path('tests').glob('*.py'):
        if args.only and path.stem not in args.only:
            continue
        print(path.name)
        if path.name.startswith('_'):
            print("Skipping")
            continue
        with open(path, "r", encoding="utf-8") as f:
            code = f.read()
            execute = "# norun" not in code
            compile = "# nocompile" not in code
            res = format_code(transpile(code, path.name, path))
        #print(res)
        name_cpp = path.with_suffix('.cpp')
        with open(name_cpp, "w", encoding="utf-8") as fcpp:
            fcpp.write(res)
        print(".cpp generated")
        if args.compile:
            continue
        execute_str = "true" if (execute and not args.generate) else "false"
        name_bin = path.with_suffix('').as_posix()
        commands = [
           f'bash -c "export PYTHONPATH=stdlib; if {execute_str}; then python3 ./{path.as_posix()}; fi"',
        ]
        if compile and (alt := environ.get("ALT_RUNNER")):
            commands.append(alt.format(name_bin=name_bin, name_cpp_posix=name_cpp.as_posix(), run_file=execute_str))
        else:
            print("no ALT_RUNNER")
        for cmd in commands:
            print(cmd)
            if system(cmd) != 0:
                print(f"Error running command: {cmd}")
                break


if __name__ == "__main__":
    run_tests()
